@@ -462,6 +462,7 @@ struct connman_device *connman_device_create(const char *node,
 	struct connman_device *device;
 	const char *str;
 	enum connman_service_type service_type;
+	connman_bool_t bg_scan;
 
 	DBG("node %s type %d", node, type);
 
@@ -471,6 +472,8 @@ struct connman_device *connman_device_create(const char *node,
 
 	DBG("device %p", device);
 
+	bg_scan = connman_setting_get_bool("BackgroundScanning");
+
 	__connman_element_initialize(&device->element);
 
 	device->element.name = g_strdup(node);
@@ -499,27 +502,20 @@ struct connman_device *connman_device_create(const char *node,
 
 	switch (type) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_GADGET:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
 		device->scan_interval = 0;
 		break;
-	case CONNMAN_DEVICE_TYPE_ETHERNET:
 	case CONNMAN_DEVICE_TYPE_WIFI:
-		device->scan_interval = 300;
-		break;
-	case CONNMAN_DEVICE_TYPE_WIMAX:
-		device->scan_interval = 0;
-		break;
-	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
-		device->scan_interval = 0;
-		break;
-	case CONNMAN_DEVICE_TYPE_GPS:
-		device->scan_interval = 0;
-		break;
-	case CONNMAN_DEVICE_TYPE_CELLULAR:
-		device->scan_interval = 0;
-		break;
-	case CONNMAN_DEVICE_TYPE_GADGET:
-		device->scan_interval = 0;
+		if (bg_scan == TRUE)
+			device->scan_interval = 300;
+		else
+			device->scan_interval = 0;
 		break;
 	}
 
@@ -787,7 +783,7 @@ int __connman_device_disconnect(struct connman_device *device)
 	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
 		struct connman_network *network = value;
 
-		if (__connman_network_get_connecting(network) == TRUE) {
+		if (connman_network_get_connecting(network) == TRUE) {
 			/*
 			 * Skip network in the process of connecting.
 			 * This is a workaround for WiFi networks serviced
@@ -1279,7 +1275,6 @@ static int device_load(struct connman_device *device)
 	GError *error = NULL;
 	gchar *identifier;
 	connman_bool_t powered;
-	int val;
 
 	DBG("device %p", device);
 
@@ -1297,12 +1292,6 @@ static int device_load(struct connman_device *device)
 		device->powered_persistent = powered;
 	g_clear_error(&error);
 
-	val = g_key_file_get_integer(keyfile, identifier,
-						"ScanInterval", &error);
-	if (error == NULL)
-		device->scan_interval = val;
-	g_clear_error(&error);
-
 done:
 	g_free(identifier);
 
@@ -1330,9 +1319,6 @@ static int device_save(struct connman_device *device)
 	g_key_file_set_boolean(keyfile, identifier,
 					"Powered", device->powered_persistent);
 
-	g_key_file_set_integer(keyfile, identifier,
-					"ScanInterval", device->scan_interval);
-
 done:
 	g_free(identifier);
 
