@@ -40,6 +40,49 @@
 
 #include "connman.h"
 
+static struct {
+	connman_bool_t bg_scan;
+} connman_settings  = {
+	.bg_scan = TRUE,
+};
+
+static GKeyFile *load_config(const char *file)
+{
+	GError *err = NULL;
+	GKeyFile *keyfile;
+
+	keyfile = g_key_file_new();
+
+	g_key_file_set_list_separator(keyfile, ',');
+
+	if (!g_key_file_load_from_file(keyfile, file, 0, &err)) {
+		connman_error("Parsing %s failed: %s", file, err->message);
+		g_error_free(err);
+		g_key_file_free(keyfile);
+		return NULL;
+	}
+
+	return keyfile;
+}
+
+static void parse_config(GKeyFile *config)
+{
+	GError *error = NULL;
+	gboolean boolean;
+
+	if (config == NULL)
+		return;
+
+	DBG("parsing main.conf");
+
+	boolean = g_key_file_get_boolean(config, "General",
+						"BackgroundScanning", &error);
+	if (error == NULL)
+		connman_settings.bg_scan = boolean;
+
+	g_clear_error(&error);
+}
+
 static GMainLoop *main_loop = NULL;
 
 static void sig_term(int sig)
@@ -63,8 +106,8 @@ static gchar *option_nodevice = NULL;
 static gchar *option_noplugin = NULL;
 static gchar *option_wifi = NULL;
 static gboolean option_detach = TRUE;
+static gboolean option_dnsproxy = TRUE;
 static gboolean option_compat = FALSE;
-static gboolean option_selftest = FALSE;
 static gboolean option_version = FALSE;
 
 static gboolean parse_debug(const char *key, const char *value,
@@ -95,10 +138,11 @@ static GOptionEntry options[] = {
 	{ "nodaemon", 'n', G_OPTION_FLAG_REVERSE,
 				G_OPTION_ARG_NONE, &option_detach,
 				"Don't fork daemon to background" },
+	{ "nodnsproxy", 'r', G_OPTION_FLAG_REVERSE,
+				G_OPTION_ARG_NONE, &option_dnsproxy,
+				"Don't enable DNS Proxy" },
 	{ "compat", 'c', 0, G_OPTION_ARG_NONE, &option_compat,
-				"Enable Network Manager compatibility" },
-	{ "selftest", 't', 0, G_OPTION_ARG_NONE, &option_selftest,
-				"Run self testing routines" },
+				"(obsolete)" },
 	{ "version", 'v', 0, G_OPTION_ARG_NONE, &option_version,
 				"Show version information and exit" },
 	{ NULL },
@@ -116,6 +160,14 @@ const char *connman_option_get_string(const char *key)
 	return NULL;
 }
 
+connman_bool_t connman_setting_get_bool(const char *key)
+{
+	if (g_str_equal(key, "BackgroundScanning") == TRUE)
+		return connman_settings.bg_scan;
+
+	return FALSE;
+}
+
 int main(int argc, char *argv[])
 {
 	GOptionContext *context;
@@ -124,6 +176,7 @@ int main(int argc, char *argv[])
 	DBusError err;
 	struct sigaction sa;
 	mode_t old_umask;
+	GKeyFile *config;
 
 #ifdef HAVE_CAPNG
 	/* Drop capabilities */
@@ -203,24 +256,14 @@ int main(int argc, char *argv[])
 
 	g_dbus_set_disconnect_function(conn, disconnect_callback, NULL, NULL);
 
-	if (option_compat == TRUE) {
-		if (g_dbus_request_name(conn, NM_SERVICE, NULL) == FALSE) {
-			fprintf(stderr, "Can't register compat service\n");
-			option_compat = FALSE;
-		}
-	}
-
 	__connman_log_init(option_debug, option_detach);
 
-	if (option_selftest == TRUE) {
-		if (__connman_selftest() < 0) {
-			connman_error("Self testing routines failed");
-			goto selftest;
-		}
-	}
-
 	__connman_dbus_init(conn);
 
+	config = load_config(CONFIGDIR "/main.conf");
+
+	parse_config(config);
+
 	__connman_storage_init();
 	__connman_element_init(option_device, option_nodevice);
 
@@ -228,13 +271,13 @@ int main(int argc, char *argv[])
 	__connman_iptables_init();
 	__connman_tethering_init();
 	__connman_counter_init();
-	__connman_ondemand_init();
-	__connman_manager_init(option_compat);
+	__connman_manager_init();
 	__connman_profile_init();
 	__connman_config_init();
 	__connman_stats_init();
+	__connman_clock_init();
 
-	__connman_resolver_init();
+	__connman_resolver_init(option_dnsproxy);
 	__connman_ipconfig_init();
 	__connman_rtnl_init();
 	__connman_task_init();
@@ -242,6 +285,7 @@ int main(int argc, char *argv[])
 	__connman_detect_init();
 	__connman_session_init();
 	__connman_timeserver_init();
+	__connman_connection_init();
 
 	__connman_plugin_init(option_plugin, option_noplugin);
 
@@ -263,6 +307,7 @@ int main(int argc, char *argv[])
 
 	__connman_plugin_cleanup();
 
+	__connman_connection_cleanup();
 	__connman_timeserver_cleanup();
 	__connman_session_cleanup();
 	__connman_detect_cleanup();
@@ -272,11 +317,11 @@ int main(int argc, char *argv[])
 	__connman_ipconfig_cleanup();
 	__connman_resolver_cleanup();
 
+	__connman_clock_cleanup();
 	__connman_stats_cleanup();
 	__connman_config_cleanup();
 	__connman_profile_cleanup();
 	__connman_manager_cleanup();
-	__connman_ondemand_cleanup();
 	__connman_counter_cleanup();
 	__connman_agent_cleanup();
 	__connman_tethering_cleanup();
@@ -287,12 +332,14 @@ int main(int argc, char *argv[])
 
 	__connman_dbus_cleanup();
 
-selftest:
 	__connman_log_cleanup();
 
 	dbus_connection_unref(conn);
 
 	g_main_loop_unref(main_loop);
 
+	if (config)
+		g_key_file_free(config);
+
 	return 0;
 }
